# Multidimensional Item Response Theory: Supplementary R code
# by Wes Bonifay

##### Introduction

# Most of the models and methods in _Multidimensional Item Response Theory_ (Bonifay, 2020) 
# are included in the _mirt_ R package (Chalmers, 2012; for more details on this extensive 
# package see <https://cran.r-project.org/web/packages/mirt/mirt.pdf>). Additional packages, 
# including _psych_ (Revelle, 2012), _sirt_ (Robitzsch, 2019), _mokken_ (van der Ark, 2018), 
# _plink_ (Weeks, 2010), and _mirtCAT_ (Chalmers, 2015) are needed for certain methods that 
# are discussed in the book. Thus, it is necessary to begin by installing these packages if 
# you have not already done so. This can be accomplished by running code such as 
# _install.packages('mirt', dependencies = TRUE)_ which instructs R to download and install 
# the _mirt_ package and any additional packages that it requires. After installation, the 
# necessary packages can be accessed via:
  
library(mirt); library(psych); library(sirt); library(mokken); library(plink); library(mirtCAT)

# The example code below is organized according to the chapter outline of the text. Whenever possible,
# the exact code used in creating the plots in the text have been included if you wish to reproduce
# and/or modify any of the figures.


##### Chapter 2: Unidimensional Item Response Theory

### Data Assumptions of UIRT

# The unidimensionality assumption can be checked in a variety of ways. 
# De Ayala (2009) recommends the use of scree plots and inspection of eigenvalues. 
# Both of these methods can be conducted (and in a much more rigorous way) by running 
# Horn's parallel analysis in the _psych_ package:

fa.parallel(data)

# Run ?fa.parallel.poly for details on this method and how the results can be used to
# determine whether your data satisfy the unidimensionality assumption. Another common 
# tool for dimensionality assessment is the DETECT procedure (Zhang & Stout, 1999), 
# which is included in the _sirt_ package:

expl.detect(data, score, nclusters)

# The code above run an exploratory DETECT analysis (while conf.detect runs a confirmatory analysis).
# The _score_ argument is an ability estimate (e.g., summed score, mean score, or IRT-scaled score) and
# _nclusters_ indicates the number of hypothesized clusters, just as in exploratory factor analysis. 
# Run ?expl.detect for further information on DETECT.

# The _mirt_ package allows for evaluation of the local independence assumption via the LD X2 (Chen
# & Thissen, 1997) and Q3 (Yen, 1984) statistics. Note that the _object_ argument in this code  
# denotes a fitted _mirt_ model, as described in more detail below. Run ?mirt::`residuals-method` for 
# detailed information and further reading on these statistics.

residuals(object, type = 'LD') #for LD X2 statistics
residuals(object, type = 'Q3') #for Q3 statistics

# Finally, the monotoncity assumption can be evaluated using the _mokken_ package for nonparametric
# IRT modeling. This package includes a list of results from a monotonocity evaluation (Junker & Sijstma,
# 2000), as well as a series of nonparametric IRT plots. f the nonparametric item response functions are
# monotonically increasing as the level of the latent trait increases, then this assumption has likely
# been satisfied by your data. Run ?check.monotonicity for more details.

monotonocity.list <- check.monotonicity(data)
plot(monotonicity.list)
summary(monotonicity.list)


### UIRT Models for Dichotomous Data

# The _mirt_ package makes it simple to estimate and evaluation unidimensional IRT models. The _mirt()_ 
# command only requires that the user inputs the data and the model. The former must consist of
# numerically ordered data with missing values coded as NA, while in UIRT, the latter is simply the
# numeral 1, indicating a single underlying latent trait. Many additional arguments can be passed to the
# _mirt()_ command, including the itemtype and guessing coefficient, as well as many technical 
# specifications regarding standard errors, estimation, prior specificaion, and other advanced topics.
# See ?mirt for a comprehensive overview of this command, including the default estimation settings. 

# The dichotomous MIRT models discussed in Chapter 2 can be estimated by running one of the following. 
# Note that the _mirt_ package will automatically detect whether your data are dichotomous or polytomous.

FourPLmod <- mirt(data, 1, itemtype = '4PL')
ThreePLmod <- mirt(data, 1, itemtype = '3PL')
TwoPLmod <- mirt(data, 1, itemtype = '2PL')
Raschmod <- mirt(data, 1, itemtype = 'Rasch')

### UIRT Models for Polytomous Data

# The polytomous MIRT models discussed in Chapter 2 can be estimated by running one of the following.
# The graded responses model (i.e., the difference model) is the default in the _mirt()_ command
# and will be estimated if you input polytomous data but do not specify an _itemtype_.

GRmod <- mirt(data, 1, itemtype = 'graded')

# The nominal, generalized partial credit, and generalized rating scale (divide-by-total) models:

NRmod <- mirt(data, 1, itemtype = 'nominal')
GPCmod <- mirt(data, 1, itemtype = 'gpcm')
GRSmod <- mirt(data, 1, itemtype = 'grsm')

### Additional UIRT Models

# Similar syntax in the _mirt_ package can be used to estimate several additional UIRT models, including 
# the ideal point model (Maydeu-Olivares, 2006) and the monotonic polynomial model (Falk & Cai, 2016):

IPmod <- mirt(data, 1, itemtype = 'ideal')
MPmod <- mirt(data, 1, itemtype = 'monopoly')

# Note that the _sirt_ package provides many rarely applied and understudied UIRT models that may 
# be of interest. Run ?sirt to access documentation regarding the available supplementary IRT models.

### After model fitting

# After fitting any of the UIRT models listed above to your data, the _mirt_ package allows you to 
# investigate a wide array of results and figures (see ?mirt documentation for full details), including:

coef(mod) #Estimated parameter coefficients
summary(mod) #Transform IRT parameters into standardized factor loadings
residuals(mod) #Evaluate local independence, as described above
M2(mod) #Overall model goodness-of-fit
itemfit(mod) #Item-level goodness-of-fit
personfit(mod) #Person-level goodness-of-fit

# The _mirt_ package also allows users to obtain a variety of useful test-level and item-level plots: 

plot(mod, type = 'score') #Expected total test score
plot(mod, type = 'infoSE') #Test information and standard error functions

itemplot(mod, item = 1, type = 'trace') #Response function of Item 1
itemplot(mod, item = 1, type = 'infoSE') #Information and standard error functions of Item 1
itemplot(mod, item = 1, type = 'score') #Expected score on Item 1 (for polytomous models)

### UIRT Estimation

# The _mirt_ package also allows users to modify the estimation method. The default method is the 
# Bock-Aitkin (1981) EM algorithm, though several alternatives are available via the _method_ argument
# within the _mirt()_ command:

BLest <- mirt(mod, 1, method = 'BL') #Joint estimation (Bock & Lieberman, 1970) 
EMest <- mirt(mod, 1, method = 'EM') #Bock-Aitkin EM algorithm

### Scoring

# Finally, the predicted IRT-scaled scores (latent trait location estimates) from a UIRT model can be 
# obtained using the _fscores_ command in the _mirt_ package, and various scoring methods are supported. 
# Run ?fscores for details on scoring.

MLEs <- fscores(mod, method = 'ML') #Maximum likelihood scores
EAPs <- fscores(mod, method = 'EAP') #Expected a posteriori scores
MAPs <- fscores(mod, method = 'MAP') #Maximum a posteriori scores

# The figures from Chapter 2 can be created by modifying the a, b, g, and u parameters in the code below:

#Figures 2.1 to 2.4 - item response and information functions
log4pl <- function(a,b,g,u) {
  theta <- seq(-3,3,.1)
  points <- matrix(0,length(theta),2)
  for (i in 1:length(theta)) {
    z <- 1.702*a*(theta[i]-b)
    prob <- g+((u-g)/(1+exp(-z)))
    points[i,1] <- theta[i]
    points[i,2] <- prob
  }
  points
}
#4PL surface
windows(6,6)
par(mar=c(5,5,5,5))
a <- 1
b <- 0
g <- 0
u <- 1
trace_log <- log4pl(a,b,g,u)
theta <- trace_log[,1]
p <- trace_log[,2]
plot(theta,p,ylim=c(0,1),type='l',lwd=2,las=1,xaxs='i',yaxs='i',yaxt='n',
     xlab=expression(theta),ylab=expression(paste("P(x=1|",theta,")")),cex.lab=1.5)

u <- .8
trace_log <- log4pl(a,b,g,u)
p <- trace_log[,2]
lines(theta,p,lwd=2,lty=2)

u <- .6
trace_log <- log4pl(a,b,g,u)
p <- trace_log[,2]
lines(theta,p,lwd=2,lty=3)
legend("topleft",legend=c('u = 1.0',"u = 0.8","u = 0.6"),cex=1.2,lwd=2,lty=c(1:3))

#4PL information
u <- 1
trace_log <- log4pl(a,b,g,u)
theta <- trace_log[,1]
p <- trace_log[,2]
info <- ((a^2)*((p-g)^2)*((u-p)^2))/(((u-g)^2)*p*(1-p)) #Magis 2013, Eq 6

windows(6,6)
par(mar=c(5,5,5,5))
plot(theta,info,type='l',lwd=2,lty=1,las=1,ylim=c(0,max(info)+max(info)/10),xaxs='i',yaxs='i',yaxt='n',
     xlab=expression(theta),ylab=NA,cex.lab=1.5)
axis(4,las=1)
mtext(expression(paste("I(",theta,")")),side=4,line=3,cex=1.5)
segments(theta[which.max(info)],max(info),theta[which.max(info)],0,col='gray50')

u <- .8
trace_log <- log4pl(a,b,g,u)
p <- trace_log[,2]
info <- ((a^2)*((p-g)^2)*((u-p)^2))/(((u-g)^2)*p*(1-p)) #Magis 2013, Eq 6
lines(theta,info,lwd=2,lty=2)
segments(theta[which.max(info)],max(info),theta[which.max(info)],0,col='gray50',lty=2)

u <- .6
trace_log <- log4pl(a,b,g,u)
p <- trace_log[,2]
info <- ((a^2)*((p-g)^2)*((u-p)^2))/(((u-g)^2)*p*(1-p)) #Magis 2013, Eq 6
lines(theta,info,lwd=2,lty=3)
segments(theta[which.max(info)],max(info),theta[which.max(info)],0,col='gray50',lty=3)

# Figure 2.5 - GRM category response and information functions

p0.grm <- function(a,b1) {
  probs <- matrix(0,length(theta),1)
  for (i in 1:length(theta)) {
    z1 <- 1.702*a*(theta[i]-b1)
    p0star <- 1
    p1star <- 1/(1+exp(-z1))
    probs[i] <- p0star-p1star
  }
  probs
}
p1.grm <- function(a,b1,b2) {
  probs <- matrix(0,length(theta),1)
  for (i in 1:length(theta)) {
    z1 <- 1.702*a*(theta[i]-b1)
    z2 <- 1.702*a*(theta[i]-b2)
    p1star <- 1/(1+exp(-z1))
    p2star <- 1/(1+exp(-z2))
    probs[i] <- p1star-p2star
  }
  probs
}
p2.grm <- function(a,b2) {
  probs <- matrix(0,length(theta),1)
  for (i in 1:length(theta)) {
    z2 <- 1.702*a*(theta[i]-b2)
    p2star <- 1/(1+exp(-z2))
    p3star <- 0
    probs[i] <- p2star-p3star
  }
  probs
}

#arbitrary parameters
a <- 1
b1 <- -1
b2 <- 1
theta <- seq(-3,3,.1)

windows(6,6)
par(mar=c(5,5,5,5))
p0trace <- p0.grm(a,b1)
plot(theta,p0trace,ylim=c(0,1),type='l',lwd=2,las=1,
     ylab=expression(paste("P(x = k| ",theta,")")),
     xlab=expression(theta),cex.lab=1.5 )

p1trace <- p1.grm(a,b1,b2)
lines(theta,p1trace,ylim=c(0,1),type='l',lty=2,lwd=2)

p2trace <- p2.grm(a,b2)
lines(theta,p2trace,ylim=c(0,1),type='l',lty=3,lwd=2)

#Information
p0.info <- function(a,b1) {
  info.0 <- matrix(0,length(theta),1)
  for (i in 1:length(theta)) {
    z1 <- 1.702*a*(theta[i]-b1)
    p0star <- 1
    p1star <- 1/(1+exp(-z1))
    info.0[i] <- p0.grm(a,b1)[i]*(a^2)*(p0star*(1-p0star)+(p1star*(1-p1star)))
  }
  info.0
}
p1.info <- function(a,b1,b2) {
  info.1 <- matrix(0,length(theta),1)
  for (i in 1:length(theta)) {
    z1 <- 1.702*a*(theta[i]-b1)
    z2 <- 1.702*a*(theta[i]-b2)
    p1star <- 1/(1+exp(-z1))
    p2star <- 1/(1+exp(-z2))
    info.1[i] <- p1.grm(a,b1,b2)[i]*(a^2)*(p1star*(1-p1star)+(p2star*(1-p2star)))
  }
  info.1
}
p2.info <- function(a,b2) {
  info.2 <- matrix(0,length(theta),1)
  for (i in 1:length(theta)) {
    z2 <- 1.702*a*(theta[i]-b2)
    p2star <- 1/(1+exp(-z2))
    p3star <- 0
    info.2[i] <- p2.grm(a,b2)[i]*(a^2)*(p2star*(1-p2star)+(p3star*(1-p3star)))
  }
  info.2
}

windows(6,6)
par(mar=c(5,5,5,5))
GRMinfo <- p0.info(a,b1)+p1.info(a,b1,b2)+p2.info(a,b2)
plot(theta,GRMinfo,type='l',lwd=2,lty=1,las=1,ylim=c(0,max(GRMinfo)+max(GRMinfo)/10),xaxs='i',yaxs='i',yaxt='n',
     xlab=expression(theta),ylab=NA,cex.lab=1.5)
axis(4,las=1)
mtext(expression(paste("I(",theta,")")),side=4,line=3,cex=1.5)

#Figure 2.6 - likelihood functions
LL <- rep(0,length(theta))
x <- rep(0,length(b)) 
for (i in 1:length(theta)){
  for (j in 1:length(b)){
    p <- exp(theta[i]-b[j])/(1+exp(theta[i]-b[j]))
    x[j] <- resp[j]*log(p)+(1-resp[j])*log(1-p)
  }
  LL[i] <- sum(x)
}
LL <- round(LL,4)

resp <- c(1,1,0,0,0)
LL <- LLfunc(resp)
windows(6,6)
par(mar=c(5,5,5,5))
plot(theta,LL,type='l',ylim=c(-10,1),lwd=2,lty=1,las=1,xaxs='i',yaxs='i',
     xlab=expression(theta),ylab="Log-likelihood",cex.lab=1.5)
abline(h=0,lty=3)
xcoord <- theta[which.max(LL)]
ycoord <- max(LL)
segments(xcoord,ycoord,xcoord,-15,col='gray50',lty=2)
segments(xcoord-1.5,ycoord,xcoord+1.5,ycoord)
mtext('Response pattern = 1 1 0 0 0',line=-1.5)

#Figure 2.7 - IRT scoring plots
score.2pl <- function(a,b,pattern,thetarange=seq(-3,3,.01)) {
  prob <- rep(0,length(thetarange))
  for (i in 1:length(a)) {
    for (j in 1:length(thetarange)) {
      z <- a[i]*(thetarange[j]-b[i])
      if (pattern[i]==1) prob[j] <- 1/(1+exp(-z))
      if (pattern[i]==0) prob[j] <- 1-(1/(1+exp(-z)))
    }
  }
  prob
}

#function for plotting EAP/MAP scores
scoreplot.2pl <- function(prior,thetarange) {
  windows(20,30)
  par(mar=c(4.2,5,0,1),mfrow = c(4,1))
  Xr <- thetarange
  
  #Plot prior
  AXr <- prior
  round(AXr,5)
  plot(thetarange,AXr,type='l',xlab=expression(theta),ylab='Prior',yaxt='n',cex.lab=2,las=1,lwd=2)
  
  #Plot tracelines
  LXr_val <- matrix(0,length(a),length(thetarange))
  plot(NULL,xlim=c(-3,3),ylim=c(0,1),xlab=expression(theta),ylab='Trace lines',yaxt='n',cex.lab=2,las=1,lwd=2)
  for (i in 1:length(a)) {
    trace <- score.2pl(a[i],b[i],pattern[i])
    lines(thetarange,trace,type='l',ylab='Trace lines',xlab=expression(theta),cex=1.5,lty=2-pattern[i],lwd=2)
    LXr_val[i,] <- trace
  }
  
  #Plot likelihood
  LXr <- rep(0,length(thetarange))
  for (i in 1:length(thetarange)){
    LXr[i] <- prod(LXr_val[1:length(a),i])
  }
  round(LXr,5)
  plot(thetarange,LXr,type='l',xlab=expression(theta),ylab='Likelihood',yaxt='n',cex.lab=2,las=1,lwd=2)
  
  #Plot posterior
  plot(thetarange,AXr*LXr,type='l',xlab=expression(theta),ylab='Posterior',yaxt='n',cex.lab=2,las=1,lwd=2)
  EAP <- sum(Xr*AXr*LXr)/sum(AXr*LXr)
  PSD <- sqrt(sum((Xr-EAP)^2*LXr*AXr)/sum(LXr*AXr))
  abline(v=EAP,lty=1)
  MAP <- thetarange[which.max(AXr*LXr)]
  abline(v=MAP,lty=2)
  legend('topleft',c(paste('EAP = ',round(EAP,2)),paste('PSD = ',round(PSD,2))),cex=1.2)
  par(mar=c(5,5,1,1),mfrow = c(1,1))
}
a <- c(1,1,1,1,1)
b <- c(-2,-1,0,1,2)
pattern <- c(1,1,0,0,0)
thetarange <- seq(-3,3,.01)
prior <- dnorm(thetarange)/sum(dnorm(thetarange)) #normal prior
scoreplot.2pl(prior,thetarange)


##### Chapter 3: MIRT Models for Dichotomous Data

# Extending the UIRT code to the multidimensional case is very straightforward in the _mirt_ package.
# Recall that the UIRT examples above specified that the _mirt()_ command included a model argument,
# which was set to 1 to indicate a single latent trait. For tests in which all items exhibit within-item 
# multidimensionality, this value can simply be changed to 2, 3, and so on (though there are practical 
# limits on the number of estimable dimensions, as discussed in the textbook). For example, _mirt(data,2)_
# will estimate, for each item, the probability of correct response or endorsement conditional on two 
# latent traits (the code for Chapter 7 will demonstrate how to specify more complex MIRT models.) The 
# example MIRT modeling code below assumes 2 underlying dimensions for relative simplicity.

### Compensatory MIRT Models

M2PLmod <- mirt(data, 2, itemtype = '2PL')
M3PLmod <- mirt(data, 2, itemtype = '3PL')
M4PLmod <- mirt(data, 2, itemtype = '4PL')
MRaschmod <- mirt(data, 2, itemtype = 'Rasch')

### Partially Compensatory MIRT Models

PC2PLmod <- mirt(data, 2, itemtype = 'PC2PL')
PC3PLmod <- mirt(data, 2, itemtype = 'PC3PL')

### Additional MIRT Models for Dichotomous Data

# To date, variable compensation MIRT models do not appear to be available in R. However, several packages
# allow for diagnostic classification modeling, including _CDM_ (Robitzsch, 2019) and _GDINA_ (Ma, 2019). 
# The latter makes DCM analysis particular accessible. See the _GDINA_ package documentation via ?GDINA to 
# learn more.

# The code below reproduces the figures from Chapter 3 of the textbook (note that some of the figures were
# not created in R, so cannot be reproduced here):

#Figure 3.3 - M2PL response surface and contour plots
theta1 <- seq(-4,4,.2)
theta2 <- seq(-4,4,.2)
a1 <- 1
a2 <- .8
b1 <- -.3
b2 <- .5
int <- -(a1*b1+a2*b2)
prob <- matrix(0,length(theta1),length(theta2))
for (i in 1:length(theta1)) {
  for (j in 1:length(theta2)) {
    z <- a1*(theta1[i]-b1)+a2*(theta2[j]-b2)
    prob[i,j] <- exp(z)/(1+exp(z))
  }
}
#Surface
windows(8,8)
par(mar=c(5,5,5,5))
persp(theta1,theta2,prob,phi=30,theta=.01,ticktype='detailed',xlab="\n??1",ylab="??2\n",zlab="\nP(x=1|??)")
#Contour plot
windows(6,6)
par(mar=c(5,5,5,5))
contour(theta1,theta2,prob,las=1,labcex=1,xlab=expression(theta[1]),cex.lab=1.5,ylab=expression(theta[2]))

#Figure 3.4 - M3PL response surface and contour plot
#M3PL
theta1 <- seq(-4,4,.2)
theta2 <- seq(-4,4,.2)
a1 <- 1
a2 <- .8
a <- c(a1,a2)
b1 <- 0
b2 <- 0
b <- c(b1,b2)
d <- a%*%b
g <- .2
prob <- matrix(0,length(theta1),length(theta2))
for (j in 1:length(theta1)) {
  for (k in 1:length(theta2)) {
    z <- a1*theta1[j]+a2*theta2[k]+d
    prob[j,k] <- g+(1-g)*(exp(z)/(1+exp(z)))
  }
}
#Surface
windows(6,6)
par(mar=c(5,5,5,5))
persp(theta1,theta2,prob,d=2,phi=30,theta=20,ticktype='detailed',xlab="\n??1",ylab="\n??2",zlab="\nP(x=1|??)",zlim=c(0,1))
#Contour plot
windows(6,6)
par(mar=c(5,5,5,5))
contour(theta1,theta2,prob,las=1,labcex=1,xlab=expression(theta[1]),cex.lab=1.5,ylab=expression(theta[2]))

#Figure 3.5 - M4PL response surface and contour plot
theta1 <- seq(-4,4,.2)
theta2 <- seq(-4,4,.2)
a1 <- 1
a2 <- .8
a <- c(a1,a2)
b1 <- 0
b2 <- 0
b <- c(b1,b2)
d <- a%*%b
g <- .2
u <- .8
prob <- matrix(0,length(theta1),length(theta2))
for (j in 1:length(theta1)) {
  for (k in 1:length(theta2)) {
    z <- a1*theta1[j]+a2*theta2[k]+d
    prob[j,k] <- g+(u-g)*(exp(z)/(1+exp(z)))
  }
}
#Surface
windows(6,6)
par(mar=c(5,5,5,5))
persp(theta1,theta2,prob,d=2,phi=30,theta=20,ticktype='detailed',xlab="\n??1",ylab="\n??2",zlab="\nP(x=1|??)",zlim=c(0,1))
#Contour plot
windows(6,6)
par(mar=c(5,5,5,5))
contour(theta1,theta2,prob,levels=seq(.1,.9,.1),las=1,labcex=1,xlab=expression(theta[1]),cex.lab=1.5,ylab=expression(theta[2]))

#Figures 3.6 and 3.7 - Partially compensatory surfaces and contour plots
theta1 <- seq(-4,4,.2)
theta2 <- seq(-4,4,.2)
a1 <- .5
a2 <- 2
b1 <- 0
b2 <- 0
ncprob <- matrix(0,length(theta1),length(theta2))
for (i in 1:length(theta1)) {
  for (j in 1:length(theta2)) {
    z1 <- a1*(theta1[i]-b1)
    z2 <- a2*(theta2[j]-b2)
    ncprob[i,j] <- (exp(z1)/(1+exp(z1)))*(exp(z2)/(1+exp(z2)))
  }
}
#Surface
windows(6,6)
par(mar=c(5,5,5,5))
persp(theta1,theta2,ncprob,d=2,phi=30,theta=20,ticktype='detailed',xlab="\n??1",ylab="\n??2",zlab="\nP(x=1|??)",zlim=c(0,1))
#Contour plots
windows(6,6)
par(mar=c(5,5,5,5))
contour(theta1,theta2,ncprob,nlevels=9,las=1,labcex=1,xlab=expression(theta[1]),cex.lab=1.5,ylab=expression(theta[2]))#,lwd=c(rep(.5,5),3,rep(.5,5)),yaxs='i',xaxs='i')


##### Chapter 4: MIRT Models for Polytomous Data

# As in the dichotomous MIRT examples above, the _mirt_ package makes it easy to estimate a variety of
# polytomous MIRT models by simply modifying the number of dimensions in the model argument. Again, the
# examples below use the model = 2 case for simplification.

MGRmod <- mirt(data, 2, itemtype = 'graded')
MGPCmod <- mirt(data, 2, itemtype = 'gpcm')
MNRmod <- mirt(data, 2, itemtype = 'nominal')

# Due to the efficient design of the _mirt_ package, most of the code from the UIRT can be used after
# fitting any of the dichotomous or polytomous MIRT models listed above. That is, coefficients, factor 
# loadings, residuals, plots, and so on can be found using _coef()_, _summary()_, _residuals()_, 
# _plot()_, _itemplot()_, etc., just as in the UIRT case. Note that Bonifay (2020) did not use the
# _mirt_ package to create most of his plots; the exact plotting code that he wrote is appended to this
# R script.

# The code below reproduces the plots from Chapter 4 of the textbook.

#Figure 4.1 - MGRM category response surfaces
p0.grm <- function(a1,a2,b11,b12) {
  prob <- matrix(0,length(theta1),length(theta2))
  for (i in 1:length(theta1)) {
    for (j in 1:length(theta2)) {
      z <- a1*(theta1[i]-b11)+a2*(theta2[j]-b12)
      p0star <- 1
      p1star <- exp(z)/(1+exp(z))
      prob[i,j] <- p0star-p1star
    }
  }
  prob
}

p1.grm <- function(a1,a2,b11,b12,b21,b22) {
  prob <- matrix(0,length(theta1),length(theta2))
  for (i in 1:length(theta1)) {
    for (j in 1:length(theta2)) {
      z0 <- a1*(theta1[i]-b11)+a2*(theta2[j]-b12)
      z1 <- a1*(theta1[i]-b21)+a2*(theta2[j]-b22)
      p0star <- exp(z0)/(1+exp(z0))
      p1star <- exp(z1)/(1+exp(z1))
      prob[i,j] <- p0star-p1star
    }
  }
  prob
}

p2.grm <- function(a1,a2,b21,b22) {
  prob <- matrix(0,length(theta1),length(theta2))
  for (i in 1:length(theta1)) {
    for (j in 1:length(theta2)) {
      z1 <- a1*(theta1[i]-b21)+a2*(theta2[j]-b22)
      p1star <- exp(z1)/(1+exp(z1))
      p2star <- 0
      prob[i,j] <- p1star-p2star
    }
  }
  prob
}

#arbitrary parameters
theta1 <- seq(-4,4,.25)
theta2 <- seq(-4,4,.25)
a1 <- 1
a2 <- 1
b11 <- -1
b12 <- -1
b21 <- 1
b22 <- 1

# create 3 surfaces pass through each other overlap
x <- theta1
y <- theta2
z1 <- p0.grm(a1,a2,b11,b12)
z2 <- p1.grm(a1,a2,b11,b12,b21,b22)
z3 <- p2.grm(a1,a2,b21,b22)

windows(6,6)
par(mar=c(5,5,5,5))
p1 <- persp(x,y,z1,phi=18,theta=40,d=2,col=rgb(1,0,0,0),zlim=c(0,1),xlab="\ntheta1",ylab="\ntheta2",zlab="\nP(x=k|theta)",ticktype='detailed', bor="white")

for(level in c("bottom", "middle", "top"))
  for(k in 1:3)
    for(i in 2:length(x))
      for(j in 2:length(y))
      {
        x.square <- c(x[i-1],x[i],x[i],x[i-1])
        y.square <- c(y[j-1],y[j-1],y[j],y[j])
        
        z1.square <- c(z1[i-1,j-1], z1[i,j-1], z1[i,j], z1[i-1,j])
        z2.square <- c(z2[i-1,j-1], z2[i,j-1], z2[i,j], z2[i-1,j])
        z3.square <- c(z3[i-1,j-1], z3[i,j-1], z3[i,j], z3[i-1,j])	
        
        others <- (1:3)[-k]
        
        myz.square <- get(paste("z",k,".square", sep=""))
        otherz.square1 <- get(paste("z",others[1],".square", sep=""))
        otherz.square2 <- get(paste("z",others[2],".square", sep=""))
        
        max.z <- max(mean(myz.square), mean(otherz.square1), mean(otherz.square2))
        min.z <- min(mean(myz.square), mean(otherz.square1), mean(otherz.square2))
        
        if(level=="bottom" & mean(myz.square) == min.z |
           level=="middle" & mean(myz.square) != min.z & mean(myz.square) != max.z |
           level=="top" & mean(myz.square) == max.z)
        {	
          polygon(trans3d(x=x.square,
                          y=y.square,
                          z=myz.square,pmat=p1), col=mycols[k])
        }
      }

#Figure 4.2 - individual category response surfaces and contour plots
windows(6,6)
par(mar=c(5,5,5,5))
persp(x,y,z1,phi=18,theta=40,d=2,zlim=c(0,1),xlab="\ntheta1",ylab="\ntheta2",zlab="\nP(x=0|theta)",ticktype='detailed',cex.lab=1.5)
windows(6,6)
par(mar=c(5,5,5,5))
contour(x,y,z1,las=1,labcex=1,xlab=expression(theta[1]),cex.lab=1.5,ylab=expression(theta[2]),lwd=seq(1,5,length=10))

windows(6,6)
par(mar=c(5,5,5,5))
persp(x,y,z2,phi=18,theta=40,d=2,zlim=c(0,1),xlab="\ntheta1",ylab="\ntheta2",zlab="\nP(x=1|theta)",ticktype='detailed',cex.lab=1.5)
windows(6,6)
par(mar=c(5,5,5,5))
contour(x,y,z2,las=1,labcex=1,xlab=expression(theta[1]),cex.lab=1.5,ylab=expression(theta[2]),lwd=seq(1,5,length=10))

windows(6,6)
par(mar=c(5,5,5,5))
persp(x,y,z3,phi=18,theta=40,d=2,zlim=c(0,1),xlab="\ntheta1",ylab="\ntheta2",zlab="\nP(x=2|theta)",ticktype='detailed',cex.lab=1.5)
windows(6,6)
par(mar=c(5,5,5,5))
contour(x,y,z3,las=1,labcex=1,xlab=expression(theta[1]),cex.lab=1.5,ylab=expression(theta[2]),lwd=seq(1,5,length=10))

#Figure 4.3 - expected score surface
score <- z1*0+z2*1+z3*2
windows(6,6)
par(mar=c(5,5,5,5))
persp(x,y,score,phi=18,theta=20,d=2,zlim=c(0,2),xlab="\ntheta1",ylab="\ntheta2",zlab="\nExpected Score",ticktype='detailed',cex.lab=1.3)
windows(6,6)
par(mar=c(5,5,5,5))
contour(x,y,score,levels=c(0,.5,1,1.5),las=1,labcex=1,xlab=expression(theta[1]),cex.lab=1.5,ylab=expression(theta[2]),lwd=seq(0,5,length=4))


##### Chapter 5: Descriptive MIRT Statistics

# The MIRT descriptive statistics that are detailed in Chapter 5 of the text are not presented by
# default in the _mirt_ package, but they can be easily obtained with a few lines of code. After fitting
# any of the MIRT models listed above, run:

MDisc <- MDISC(mod)
Mdiff <- MDIFF(mod)

# The direction of measurement does not seem to be included in the current version of the _mirt_ 
# package, but can be computed directly from the formulas in the textbook:

a <- c(a1,a2) #plug in univariate discrimination parameters a1 and a2
Aj <- sqrt(sum(a^2)) #MDISC
angle1 <- acos(a1/Aj)*(180/pi) #angle wrt theta1
angle2 <- acos(a2/Aj)*(180/pi) #angle wrt theta2

# MIRT information surfaces can easily be plotted using the _itemplot()_ command after fitting a MIRT
# model, just as in the UIRT examples above:

itemplot(mod, item = 1, type = 'info') #Multidimensional information of Item 1
plot(mod, type = 'info') #Multidimensional test information

# The code below reproduces the figures that are included in Chapter 5 of the text.

# Figure 5.2 - rotated surface
windows(6,6)
par(mar=c(5,5,5,5))
persp(theta1,theta2,prob,d=2,phi=30,theta=20,ticktype='detailed',xlab="\ntheta1",ylab="\ntheta2",zlab="\nP(x=1|theta)")
windows(6,6)
par(mar=c(5,5,5,5))
persp(theta1,theta2,prob,d=2,phi=30,theta=40,ticktype='detailed',xlab="\ntheta1",ylab="\ntheta2",zlab="\nP(x=1|theta)")
windows(6,6)
par(mar=c(5,5,5,5))
persp(theta1,theta2,prob,d=2,phi=30,theta=-70,ticktype='detailed',xlab="\ntheta1",ylab="\ntheta2",zlab="\nP(x=1|theta)")
windows(6,6)
par(mar=c(5,5,5,5))
persp(theta1,theta2,prob,d=2,phi=30,theta=-130,ticktype='detailed',xlab="\ntheta1",ylab="\ntheta2",zlab="\nP(x=1|theta)")

# Figure 5.3 - conditional slices (compensatory)
windows(4,4)
theta2_slice <- 2 #modify value of 2nd dimension
z1 <- a1*(theta1-b1)+a2*(theta2_slice-b2)
prob1 <- exp(z1)/(1+exp(z1))
plot(theta2,prob1,type='l',ylim=c(0,1),las=1,xlim=c(-4,4),ylab='P(x = 1)',xlab=expression(theta[1]),lwd=3)
key <- as.expression(bquote(paste(theta[1],' | ',theta[2])==.(theta2_slice)))
legend("bottomright",legend=key,cex=1.5,bty='n')

# Figure 5.4 - conditional slices (partially compensatory)
theta1 <- seq(-4,4,.2)
theta2 <- seq(-4,4,.2)
a1 <- 1
a2 <- .8
b1 <- -.2
b2 <- .5
ncprob <- matrix(0,length(theta1),length(theta2))
for (i in 1:length(theta1)) {
  for (j in 1:length(theta2)) {
    z1 <- a1*(theta1[i]-b1)
    z2 <- a2*(theta2[j]-b2)
    ncprob[i,j] <- (exp(z1)/(1+exp(z1)))*(exp(z2)/(1+exp(z2)))
  }
}
windows(6,6)
par(mar=c(5,5,5,5))
persp(theta1,theta2,ncprob,d=2,phi=30,theta=30,ticktype='detailed',xlab="\ntheta1",ylab="\ntheta2",zlab="\nP(x=1|theta)")
windows(4,4)
theta2_slice1 <- 2
z1 <- a1*(theta1-b1)
z2 <- a2*(theta2_slice1-b2)
ncprob1 <- (exp(z1)/(1+exp(z1)))*(exp(z2)/(1+exp(z2)))
plot(theta2,ncprob1,type='l',ylim=c(0,1),las=1,xlim=c(-4,4),ylab='P(x = 1)',xlab=expression(theta[1]),lwd=3)
key <- as.expression(bquote(paste(theta[1],' | ',theta[2])==.(theta2_slice1)))
legend("topleft",legend=key,cex=1.5,bty='n')

# Figures 5.5 and 5.7 - contour plots
windows(6,6)
par(mar=c(5,5,5,5))
contour(theta1,theta2,prob,las=1,labcex=1,xlab=expression(theta[1]),cex.lab=1.5,ylab=expression(theta[2]),lwd=c(rep(.5,5),3,rep(.5,5)))
a <- c(a1,a2)
Aj <- sqrt(sum(a^2)) #hypotenuse
angle1 <- acos(a1/Aj) #angle wrt theta1
angle2 <- acos(a2/Aj) #angle wrt theta2
b1_tail <- b1+(b2-b1*tan(angle1))*cos(angle2)*cos(angle1)
b2_tail <- b1*tan(angle1)+(b2-b1*tan(angle1))*(cos(angle2)^2)
arrows(b1_tail,b2_tail,b1_tail+Aj*cos(angle1),b2_tail+Aj*sin(angle1),lwd=3,length=.2)
abline(0,a2/a1,lty=2)
abline(h=0,col='gray')
abline(v=0,col='gray')
segments(0,0,b1_tail,b2_tail,lwd=8,lend=1)
points(b1,b2,pch='+',cex=3,bg='white',lwd=2)
points(b1_tail,b2_tail,pch=21,cex=2,bg='white',lwd=2)

# Figures 5.9 and 5.10- information plots
info <- Aj^2*prob*(1-prob)
windows(6,6)
par(mar=c(5,5,5,5))
persp(theta1,theta2,info,d=2,phi=30,theta=25,ticktype='detailed',xlab="\ntheta1",ylab="\ntheta2",zlab="\nI(theta)")
windows(6,6)
par(mar=c(5,5,5,5))
contour(theta1,theta2,info,las=1,labcex=1,xlab=expression(theta[1]),cex.lab=1.5,ylab=expression(theta[2]),lwd=seq(.5,5,.5))
b1_tail <- b1+(b2-b1*tan(angle1))*cos(angle2)*cos(angle1)
b2_tail <- b1*tan(angle1)+(b2-b1*tan(angle1))*(cos(angle2)^2)
arrows(b1_tail,b2_tail,b1_tail+Aj*cos(angle1),b2_tail+Aj*sin(angle1),lwd=5,length=.2)
abline(0,a2/a1,lty=2)
abline(h=0,col='gray')
abline(v=0,col='gray')
points(b1_tail,b2_tail,pch=21,cex=2,bg='white',lwd=2)

# Figure 5.11 - polytomous MIRT response surface
p0.grm <- function(a1,a2,b11,b12) {
  prob <- matrix(0,length(theta1),length(theta2))
  for (i in 1:length(theta1)) {
    for (j in 1:length(theta2)) {
      z <- a1*(theta1[i]-b11)+a2*(theta2[j]-b12)
      p0star <- 1
      p1star <- exp(z)/(1+exp(z))
      prob[i,j] <- p0star-p1star
    }
  }
  prob
}

p1.grm <- function(a1,a2,b11,b12,b21,b22) {
  prob <- matrix(0,length(theta1),length(theta2))
  for (i in 1:length(theta1)) {
    for (j in 1:length(theta2)) {
      z0 <- a1*(theta1[i]-b11)+a2*(theta2[j]-b12)
      z1 <- a1*(theta1[i]-b21)+a2*(theta2[j]-b22)
      p0star <- exp(z0)/(1+exp(z0))
      p1star <- exp(z1)/(1+exp(z1))
      prob[i,j] <- p0star-p1star
    }
  }
  prob
}

p2.grm <- function(a1,a2,b21,b22) {
  prob <- matrix(0,length(theta1),length(theta2))
  for (i in 1:length(theta1)) {
    for (j in 1:length(theta2)) {
      z1 <- a1*(theta1[i]-b21)+a2*(theta2[j]-b22)
      p1star <- exp(z1)/(1+exp(z1))
      p2star <- 0
      prob[i,j] <- p1star-p2star
    }
  }
  prob
}

#arbitrary parameters
theta1 <- seq(-4,4,.25)
theta2 <- seq(-4,4,.25)
a1 <- 1
a2 <- 1
b11 <- -1
b12 <- -1
b21 <- 1
b22 <- 1


# create 3 surfaces pass through each other overlap
x <- theta1
y <- theta2
z1 <- p0.grm(a1,a2,b11,b12)
z2 <- p1.grm(a1,a2,b11,b12,b21,b22)
z3 <- p2.grm(a1,a2,b21,b22)

windows(6,6)
par(mar=c(5,5,5,5))
p1 <- persp(x,y,z1,phi=18,theta=40,d=2,col=rgb(1,0,0,0),zlim=c(0,1),xlab="\ntheta1",ylab="\ntheta2",zlab="\nP(x=k|theta)",ticktype='detailed', bor="white")

for(level in c("bottom", "middle", "top"))
  for(k in 1:3)
    for(i in 2:length(x))
      for(j in 2:length(y))
      {
        x.square <- c(x[i-1],x[i],x[i],x[i-1])
        y.square <- c(y[j-1],y[j-1],y[j],y[j])
        
        z1.square <- c(z1[i-1,j-1], z1[i,j-1], z1[i,j], z1[i-1,j])
        z2.square <- c(z2[i-1,j-1], z2[i,j-1], z2[i,j], z2[i-1,j])
        z3.square <- c(z3[i-1,j-1], z3[i,j-1], z3[i,j], z3[i-1,j])	
        
        others <- (1:3)[-k]
        
        myz.square <- get(paste("z",k,".square", sep=""))
        otherz.square1 <- get(paste("z",others[1],".square", sep=""))
        otherz.square2 <- get(paste("z",others[2],".square", sep=""))
        
        max.z <- max(mean(myz.square), mean(otherz.square1), mean(otherz.square2))
        min.z <- min(mean(myz.square), mean(otherz.square1), mean(otherz.square2))
        
        if(level=="bottom" & mean(myz.square) == min.z |
           level=="middle" & mean(myz.square) != min.z & mean(myz.square) != max.z |
           level=="top" & mean(myz.square) == max.z)
        {	
          polygon(trans3d(x=x.square,
                          y=y.square,
                          z=myz.square,pmat=p1), col=mycols[k])
        }
      }
slice <- -2
yslice <- which(y==slice)
windows(4,4)
plot(x,z1[,yslice],type='l',ylim=c(0,1.1),las=1,xlim=c(-4,4),ylab='P(x = k)',xlab=expression(theta[1]),lwd=2)
lines(x,z2[,yslice],lwd=2,lty=2)
lines(x,z3[,yslice],lwd=2,lty=3)
key <- as.expression(bquote(paste(theta[1],' | ',theta[2])==.(slice)))
legend("top",legend=key,cex=1.5,bty='n')

# Figure 5.12 - expected score surface
score <- z1*0+z2*1+z3*2
windows(6,6)
par(mar=c(5,5,5,5))
persp(x,y,score,phi=18,theta=20,d=2,zlim=c(0,2),xlab="\ntheta1",ylab="\ntheta2",zlab="\nExpected Score",ticktype='detailed',cex.lab=1.3)
windows(6,6)
par(mar=c(5,5,5,5))
contour(x,y,score,levels=c(0,.5,1,1.5),las=1,labcex=1,xlab=expression(theta[1]),cex.lab=1.5,ylab=expression(theta[2]),lwd=seq(0,5,length=4))

# Figure 5.13 - polytomous information
#Information
Ai <- sqrt(a1^2+a2^2)
info <- (Ai^2)*((z1*(1-z1))+(z2*(1-z2))+((z3*(1-z3))))
windows(6,6)
par(mar=c(5,5,5,5))
persp(x,y,info,phi=18,theta=20,d=2,zlim=c(0,max(info)),xlab="\ntheta1",ylab="\ntheta2",zlab="\nI(theta)",ticktype='detailed',cex.lab=1.3)
windows(6,6)
par(mar=c(5,5,5,5))
contour(x,y,info,levels=seq(0,max(info),.1),las=1,labcex=1,xlab=expression(theta[1]),cex.lab=1.5,ylab=expression(theta[2]),lwd=seq(0,5,.3))


##### Chapter 6: Item factor structures

# In the previous chapters, the MIRT code estimates within-item multidimensionality through the argument _model = 2_. Between-item 
# multidimensionality requires more complex coding via the _mirt.model_ command. This command reuires the user to define a confirmatory 
# model as a string. This string is passed to an object, which is then included in the _mirt()_ command as a replacement for the single
# values (1, 2, etc.) that were used earlier. Many user specifications (constraints, priors, means, covariances) can be included in this 
# string - see ?mirt.model for details and examples. In addition, the _bfactor()_ command simplifies model specification of the two-tier,
# bifactor, and testlet models that are discussed in Chapter 6. Using _bfactor()_, the user only needs to specify the specific factor 
# loadings as a vector. For example, the vector _c(1,1,1,2,2,2,3,3,3)_ indicates that the first three items load on specific factor
# 1, the next three load on specific factor 2, and the last three load on specific factor 3. The following will run a bifactor model:

specific <- c(1,1,1,2,2,2,3,3,3)
mod <- bfactor(data, specific)

# For the two-tier model, a model string must be included to indicate how the items load on the primary dimensions:
specific <- c(1,1,1,2,2,2,3,3,3)
model <- 'G = 1-5
          G = 6-9
          COV = G1*G2'
mod <- bfactor(data, specific, model)

# For the testlet model, the user must apply equality constraints across the discrimination parameters, as discussed
# in Chapter 6:

specific <- c(1,1,1,2,2,2,3,3,3)
model <- 'G = 1-9
CONSTRAIN = (1, a1, a2), (2, a1, a2), (3, a1, a2),
(4, a1, a3), (5, a1, a3), (6, a1, a3),
(7, a1, a4), (8, a1, a4), (9, a1, a4)
COV = S1*S1, S2*S2, S3*S3'
mod <- bfactor(data, specific, model)

# The figures published in Chapter 6 of the textbook were created in Microsoft Word and cannot be easily reproduced
# using R code.


##### Chapter 7: Estimation in MIRT Models

# Several of the MIRT estimation methods presented in Chapter 7 can be included as a _method_ argument within a _mirt()_ 
# command. The adaptive quadrature method does not seem to be currently available, but more sophisticated methods such
# Monte Carlo EM and Metropolis-Hastings Robbins-Monro estimation are possible:

mod <- mirt(data, 2, method = 'MCEM') #Monte Carlo EM estimation
mod <- mirt(data, 2, method = 'MHRM') #Metropolis-Hasting Robbins-Monro estimation

# The ?mirt documentation includes an important note: "The 'EM' is generally effective with 1-3 factors, but methods 
# such as the 'QMCEM', 'MCEM', 'SEM', or 'MHRM' should be used when the dimensions are 3 or more." Full Bayesian
# estimation would require a Bayesian R package or standalone software like JAGS, winBUGS, or similar. Note also 
# the _mirt()_ command accepts a _technical_ argument, which is a list of advanced estimation specifications 
# (number of EM or MH-RM cycles, quadrature points, burn in cycles, tuning parameters, and so forth). See ?mirt 
# documentation for details on the various technical parameters that can be modified.

# The code below was used to create the figures from Chapter 7:

#Figure 7.1
Lfunc <- function(resp) {
  prob <- matrix(0,length(theta1),length(theta2))
  L <- matrix(0,length(theta1),length(theta2))
  x <- rep(0,length(a1)) 
  for (i in 1:length(theta1)){
    for (j in 1:length(theta2)){
      for (k in 1:length(a1)){
        for(l in 1:length(a2)){
          for (m in 1:length(d)){
            z <- (a1[k]*theta1[i])+(a2[l]*theta2[j])+d[m]
            p <- exp(z)/(1+exp(z))
            x[k] <- (p^resp[k])*((1-p)^(1-resp[k]))          
          }
        }
      }
      L[i,j] <- prod(x)
    }
  }
  L
}

Lfunc <- function(resp) {
  prob <- matrix(0,length(theta1),length(theta2))
  L <- matrix(0,length(theta1),length(theta2))
  x <- rep(0,length(a1)) 
  for (i in 1:length(theta1)){
    for (j in 1:length(theta2)){
      for (k in 1:length(a1)){
        z <- (a1[k]*theta1[i])+(a2[k]*theta2[j])+d[k]
        p <- exp(z)/(1+exp(z))
        x[k] <- (p^resp[k])*((1-p)^(1-resp[k]))          
      }
      L[i,j] <- prod(x)
    }
  }
  L
}
theta1 <- seq(-4,4,.25)
theta2 <- seq(-4,4,.25)

#1 item correct
a1 <- c(1)#,1,2)
a2 <- c(.5)#,2,.1)
d <- c(0)#,-1,.5)
resp <- c(0)#,0,0)
L <- Lfunc(resp)
#persp
windows(6,6)
par(mar=c(5,5,5,5))
persp(theta1,theta2,L,d=2,phi=30,theta=20,ticktype='detailed',xlab="\nθ1",ylab="\nθ2",zlab="\nLikelihood",las=1)
#contour
windows(6,6)
contour(theta1,theta2,matrix(0,length(theta1),length(theta2)),nlevels=20,las=1,lwd=seq(1,5,length=20),
        xlab="\nθ1",ylab="\nθ2")
axis(1,at=seq(-4,4,1),labels=F)
abline(h=seq(-4,4,1),lty=3,col='grey80')
abline(v=seq(-4,4,1),lty=3,col='grey80')
abline(h=0)
abline(v=0)
contour(theta1,theta2,L,nlevels=20,las=1,lwd=seq(1,5,length=20),labcex=.8,add=T)

#1 correct, 2 incorrect
a1 <- c(1,.5)#,2)
a2 <- c(.5,1.5)#,.1)
d <- c(0,-1)#,.5)
resp <- c(0,1)#,0)
L <- Lfunc(resp)
contour(theta1,theta2,L,nlevels=20,las=1,lwd=seq(1,5,length=20),labcex=.8)
#persp
windows(6,6)
par(mar=c(5,5,5,5))
persp(theta1,theta2,L,d=2,phi=30,theta=20,ticktype='detailed',xlab="\nθ1",ylab="\nθ2",zlab="\nLikelihood",las=1)
#contour
windows(6,6)
contour(theta1,theta2,matrix(0,length(theta1),length(theta2)),nlevels=20,las=1,lwd=seq(1,5,length=20),
        xlab="\nθ1",ylab="\nθ2")
axis(1,at=seq(-4,4,1),labels=F)
abline(h=seq(-4,4,1),lty=3,col='grey80')
abline(v=seq(-4,4,1),lty=3,col='grey80')
abline(h=0)
abline(v=0)
contour(theta1,theta2,L,nlevels=20,las=1,lwd=seq(1,5,length=20),labcex=.8,add=T)


#1 correct, 2 incorrect, 3 incorrect
a1 <- c(1,.5,2)
a2 <- c(.5,1.5,.1)
d <- c(0,-1,.5)
resp <- c(0,1,1)
L <- Lfunc(resp)
contour(theta1,theta2,L,nlevels=20,las=1,lwd=seq(1,5,length=20),labcex=.8)
#persp
windows(6,6)
par(mar=c(5,5,5,5))
persp(theta1,theta2,L,d=2,phi=30,theta=20,ticktype='detailed',xlab="\nθ1",ylab="\nθ2",zlab="\nLikelihood",las=1)
#contour
windows(6,6)
contour(theta1,theta2,matrix(0,length(theta1),length(theta2)),nlevels=20,las=1,lwd=seq(1,5,length=20),
        xlab="\nθ1",ylab="\nθ2")
axis(1,at=seq(-4,4,1),labels=F)
abline(h=seq(-4,4,1),lty=3,col='grey80')
abline(v=seq(-4,4,1),lty=3,col='grey80')
abline(h=0)
abline(v=0)
contour(theta1,theta2,L,nlevels=20,las=1,lwd=seq(1,5,length=20),labcex=.8,add=T)

##### Chapter 8: MIRT Model Diagnostics and Evaluation

# As mentioned in the earlier code for checking UIRT assumptions, dimensionality assessment can be accomplished
# using various R packages. Exploratory and confirmatory DETECT analyses can be conducted in the _sirt_ package 
# using the commands:

expl.detect(data, score, nclusters)
conf.detect(data, score, itemclusters)

# The _score_ argument is an ability estimate (e.g., summed score, mean score, or IRT-scaled score)._nclusters_ 
# indicates the number of hypothesized clusters, just as in exploratory factor analysis, while _itemcluster_ 
# specifies the associated cluster of each item. Run ?expl.detect, ?conf.detect, or ?detect.index for further 
# information on running the DETECT procedure and interpreting the results.

# Test-level fit can be assessed via the _M2()_ command after fitting a model in _mirt_. This command yields 
# the M2 statistic, its degrees of freedom and p-value, and the RMSEA, CFI, and TLI based on the M2 statistic.

M2(mod)

# Item-level fit can be assessed via the _itemfit()_ command after fitting a model in _mirt_. The _fit_stats_
# argument provides many item fit statistics, including the S_X2 statistic described in Chapter 8. See ?itemfit 
# for details and example code.

itemfit(mod, fit_stats = 'S_X2')

# Nested models can be compared in the _mirt_ packages using the _anova_ command. To compare models, users must
# run two competing _mirt_ models and then include them in the _anova_ code. This will yield the likelihood
# ratio test, AIC, BIC, and other relative fit statistics. In addition, the _extract.mirt_ command can be used
# to extract and compare additional fit statistics. See ?extract.mirt for details.

anova(mod1, mod2)
extract.mirt(mod, 'DIC')

# The figure in Chapter 8 was created in Microsoft Word and cannot be easily reproduced in R.


##### Chapter 9: MIRT Applications

# Multidimensional linking and equating can be accomplished using the _plink_ package (Weeks, 2010). The 
# _plink()_ command takes user-specified IRT parameters from two groups and links them via least squares,
# Haebera and Stocking-Lord methods, or Procrustean transformations, as described in Chapter 9. Specifically,
# the ?plink documentation recommends that the _dilation_ argument should be set to 'orth.vd' to estimate the
# multidimensional linking constants via and orthogonal Procrustes method with variable dilations. Many more
# details and examples of R code are included in the ?plink documentation.

library(plink)
results <- plink(IRTpars, dilation = 'orth.vd')
summary(results) #summary of linking results
link.pars(results) #extract rescaled item parameters

# Multidimensional DIF can be evaluated using the _DIF_ command after fitting a multigroup model in the 
# _mirt_ package. Users must first run a multigroup IRT analysis using the _multipleGroup_ command, which 
# requires specification of group membership:

MGmod <- multiGroup(data, 2, group)

# The _DIF_ command can then be used to conduct a DIF analysis. This requires the user to specify which
# parameters to inspect for DIF, and provides many options for the exact type of DIF analysis to perform.
# See ?DIF for more details.

DIF(MGmod, which.par = 'a1') #inspect for DIF regarding the first discrimination parameter

# Finally, computerized adaptive testing can be carried out using the _mirtCAT_ packages (Chalmers, 2015).
# This package include a shiny HTML interface that allows users to create and administer adaptive (and non-
# adaptive) tests. The _mirtCAT_ code is too complex to present here, but readers will find comprehensive
# documentation and example code by accessing the ?mirtCAT documentation.

# The figures in Chapter 9 were created in Microsoft Word and cannot be easily reproduced in R.
